from sympy import *

t = Symbol('t', real=True, positive=True)
h = Symbol('h', real=True, positive=True)
tn = Symbol('t_n', real=True, positive=True)
tnm2 = tn - 2*h
tnm1 = tn - h
tnp1 = tn + h

fnm2 = Symbol('f_{n-2}', real=True)
fnm1 = Symbol('f_{n-1}', real=True)
fn = Symbol('f_n', real=True)
fnp1 = Symbol('f_{n+1}', real=True)
yn = Symbol('y_n', real=True)
ynp1 = Symbol('y_{n+1}', real=True)

points_order_1 = [ (tnp1, fnp1) ]
points_order_2 = [ (tn, fn), (tnp1, fnp1) ]
points_order_3 = [ (tnm1, fnm1), (tn, fn), (tnp1, fnp1) ]
points_order_4 = [ (tnm2, fnm2), (tnm1, fnm1), (tn, fn), (tnp1, fnp1) ]

def lagrange_basis (time, points):
    acc = 1
    for point in points:
        if (time != point[0]):
           acc = acc * (t - point[0])/(time - point[0])
        else:
            acc = point[1]*acc
    return acc

def lagrange (points):
    acc = 0
    for point in points:
        acc = acc + lagrange_basis (point[0], points)
    return acc

def build_adams (points):
    pl = lagrange(points)
    return simplify(integrate(pl, (t, tn, tnp1)))

print ("## Order 1")
formula1 = build_adams (points_order_1)
print (latex(Eq(ynp1,yn + formula1)))
print ()
print ("## Order 2")
formula2 = build_adams (points_order_2)
print (latex(Eq(ynp1,yn + formula2)))
print ()
print ("## Order 3")
formula3 = build_adams (points_order_3)
print (latex(Eq(ynp1,yn+formula3)))
print ()
print ("## Order 4")
formula4 = build_adams (points_order_4)
print (latex(Eq(ynp1,yn+formula4)))


# fnm5 = Symbol('f_{n-5}', real=True)
# fnm4 = Symbol('f_{n-4}', real=True)
# fnm3 = Symbol('f_{n-3}', real=True)
# tnm5 = tn - 5*h
# tnm4 = tn - 4*h
# tnm3 = tn - 3*h
# points_order_5 = [ (tnm3, fnm3), (tnm2, fnm2), (tnm1, fnm1), (tn, fn), (tnp1, fnp1) ]
# points_order_6 = [ (tnm4, fnm4),  (tnm3, fnm3), (tnm2, fnm2), (tnm1, fnm1), (tn, fn), (tnp1, fnp1) ]
# print ()
# print ("## Order 5")
# print (yn+build_adams (points_order_5))
# print ()
# print ("## Order 6")
# print (yn + build_adams (points_order_6)
# )
