from sympy import *

t = Symbol('t', real=True, positive=True)
h = Symbol('h', real=True, positive=True)
tn = Symbol('t_n', real=True, positive=True)
tnm5 = tn - 5*h
tnm4 = tn - 4*h
tnm3 = tn - 3*h
tnm2 = tn - 2*h
tnm1 = tn - h
tnp1 = tn + h

ynm5 = Symbol('y_{n-5}', real=True)
ynm4 = Symbol('y_{n-4}', real=True)
ynm3 = Symbol('y_{n-3}', real=True)
ynm2 = Symbol('y_{n-2}', real=True)
ynm1 = Symbol('y_{n-1}', real=True)
yn = Symbol('y_n', real=True)
ynp1 = Symbol('y_{n+1}', real=True)
fnp1 = Symbol('f_{n+1}', real=True)

points_order_1 = [ (tn, yn), (tnp1, ynp1) ]
points_order_2 = [ (tnm1, ynm1), (tn, yn), (tnp1, ynp1) ]
points_order_3 = [ (tnm2, ynm2), (tnm1, ynm1), (tn, yn), (tnp1, ynp1) ]
points_order_4 = [ (tnm3, ynm3), (tnm2, ynm2), (tnm1, ynm1), (tn, yn), (tnp1, ynp1) ]
points_order_5 = [ (tnm4, ynm4), (tnm3, ynm3), (tnm2, ynm2), (tnm1, ynm1), (tn, yn), (tnp1, ynp1) ]
points_order_6 = [ (tnm5, ynm5), (tnm4, ynm4), (tnm3, ynm3), (tnm2, ynm2), (tnm1, ynm1), (tn, yn), (tnp1, ynp1) ]

def lagrange_basis (time, points):
    acc = 1
    for point in points:
        if (time != point[0]):
           acc = acc * (t - point[0])/(time - point[0])
        else:
            acc = point[1]*acc
    return acc

def lagrange (points):
    acc = 0
    for point in points:
        acc = acc + lagrange_basis (point[0], points)
    return acc

def build_bdf (points):
    pl = lagrange(points)
    return simplify(pl.diff(t).subs(t, tnp1))

print ("## Order 1")
formula1 = build_bdf (points_order_1)
print (latex(Eq(fnp1, formula1)))
print ()
print ("## Order 2")
formula2 = build_bdf (points_order_2)
print (latex(Eq(fnp1,formula2)))
print ()
print ("## Order 3")
formula3 = build_bdf (points_order_3)
print (latex(Eq(fnp1,formula3)))
print ()
print ("## Order 4")
formula4 = build_bdf (points_order_4)
print (latex(Eq(fnp1,formula4)))
print ()
print ("## Order 5")
print (latex(Eq(fnp1,build_bdf (points_order_5))))
print ()
print ("## Order 6")
print (latex(Eq(fnp1,build_bdf (points_order_6))))

def build_bdf_der (points):
    pl = lagrange(points)
    f1 = simplify(pl.diff(t).subs(t, tn))
    f2 = simplify(pl.diff(t,2).subs(t, tn))
    f3 = simplify(pl.diff(t,3).subs(t, tn))
    print (latex(f1))
    print (latex(f2))
    print (latex(f3))

points_order_4_der = [ (tnm3, ynm3), (tnm2, ynm2), (tnm1, ynm1), (tn, yn) ]
build_bdf_der (points_order_4_der)
