import numpy as np
import matplotlib.pyplot as plt

def heun_one_step (f, t, y, h):
    y1 = y + h * f(t, y)
    return y + h * 0.5 * ( f(t, y) + f(t+h, y1))

def solve (f, t0, y0, tend, nsteps):
    h = (tend - t0) / nsteps; y = []
    ynm2 = y0
    ynm1 = heun_one_step (f, t0+h, ynm2, h)
    yn = heun_one_step (f, t0+2*h, ynm1, h)
    fnm2 = f(t0, ynm2)
    fnm1 = f(t0+h, ynm1)
    y.append(ynm2); y.append(ynm1)
    time = np.linspace(t0+2*h, tend, nsteps-2)
    for t in time:
        y.append(yn)
        fn = f(t, yn)
        yn = yn + h / 12.0 * (23.0 * fn - 16.0 * fnm1 + 5.0 * fnm2)
        fnm2 = fnm1
        fnm1 = fn
    return [ np.linspace(t0, tend, nsteps), y]

def dynamic (t, y):
    return np.array([-y[1], y[0]])

[t, y] = solve (dynamic, 0.0, np.array([1., 0.]), 2*np.pi*10, 500)

print (len(y))
fig, ax = plt.subplots()
plt.plot(t, y)

plt.xlabel('time')
plt.ylabel('Solution y')
plt.title("Simple Plot")
plt.legend()
plt.show()
